
/*
   Project:     SOFTSUSY 1.2
   File:        susy.h
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   Description: All display and set commands apart from (DoubleVector &) 
                involved with running to different energy scales have been
                explicitly named.

   $Log: susy.h,v $
   Revision 1.2  2001/07/18 14:42:51  allanach
   Added proper header info

  */

#ifndef SUSY_H
#define SUSY_H

/* Contains all supersymmetric RPCMSSM parameters and RGEs for interpolating
   them (In MSSM domain) */
#include<iostream.h>
#include<math.h>
#include<fstream.h>
#include "lowe.h"
#include "utils.h"

typedef enum {YU=1, YD, YE} yukawa;

// Contains data needed in beta function calculation
struct sBrevity
{
  DoubleMatrix dt, ut, et, u2, d2, e2, u2t, e2t, d2t;
  DoubleVector  gsq, g3, g4;
  double uuT, ddT, eeT;
  DoubleMatrix u1, d1, e1;

  sBrevity();
  sBrevity(const sBrevity &);
  const sBrevity & operator=(const sBrevity &);
  
  void calculate(const DoubleMatrix &, const DoubleMatrix &, const
		 DoubleMatrix &, const DoubleVector  &);
};

inline sBrevity::sBrevity()
  : dt(3, 3), ut(3, 3), et(3, 3), u2(3, 3), 
    d2(3, 3), e2(3, 3), u2t(3, 3), 
    e2t(3, 3), d2t(3, 3), gsq(1, 3), g3(1, 3), g4(1, 3), uuT(0.0),
    ddT(0.0), eeT(0.0), u1(3, 3), d1(3, 3), e1(3, 3)
{}

inline sBrevity::sBrevity(const sBrevity &s)
  : dt(s.dt), ut(s.ut), et(s.ut), u2(s.u2), d2(s.d2),
    e2(s.e2), u2t(s.u2t),
    e2t(s.e2t), d2t(s.d2t), gsq(s.gsq), g3(s.g3), g4(s.g4),
    uuT(s.uuT), ddT(s.ddT), eeT(s.eeT), u1(s.u1), d1(s.d1), e1(s.e1) 
{}

/* ------------------SUSY starts here ---------------------*/
class MssmSusy: public RGE
{
private:
  DoubleMatrix u, d, e; // Yukawa matrices
  DoubleVector g; // Gauge couplings
  double smu, tanb; // Bilinear Higgs parameter
  
public:
  MssmSusy();
  MssmSusy(const MssmSusy &);
  MssmSusy(const DoubleMatrix &, const DoubleMatrix &, const DoubleMatrix
	    &, const DoubleVector  &, double, double, double, int, int);
  virtual ~MssmSusy() {};
  
  const MssmSusy & operator=(const MssmSusy & s);
  
  void setTrilinears(const MssmSusy & s);
  void setYukawaElement(yukawa, int, int, double);
  void setYukawaMatrix(yukawa, const DoubleMatrix &);
  void setGaugeCoupling(int, double);
  void setAllGauge(const DoubleVector  &);
  void setSusyMu(double);
  void set(const DoubleVector &);
  void setTanb(double);
  
  double displayYukawaElement(yukawa, int, int) const;
  DoubleMatrix displayYukawaMatrix(yukawa) const;
  double displayGaugeCoupling(int) const;
  DoubleVector  displayGauge() const;
  double displaySusyMu() const;
  DoubleVector display() const;
  double displayTanb() const;
  
  void getMasses(QedQcd &, double vev) const;
  void quarkMixing(const DoubleMatrix &, int);
  void setDiagYukawas(const QedQcd &, double vev);
  void readIn(const QedQcd &, char [], double t, double vev);
  void getQuarkMixedYukawas(const QedQcd & r, const DoubleMatrix &
				    CKM, int mix, double vev);
  DoubleVector beta() const;
  MssmSusy beta(sBrevity &) const;
  void getOneLpAnom(DoubleMatrix &, DoubleMatrix &, DoubleMatrix &,
		       DoubleMatrix &, DoubleMatrix &, double &, double &,
		       sBrevity &) const; 
  void getTwoLpAnom(DoubleMatrix &, DoubleMatrix &, DoubleMatrix &,
		       DoubleMatrix &, DoubleMatrix &, double &, double &,
		       sBrevity &) const; 
  
  void anomalousDimension(DoubleMatrix &, DoubleMatrix &, DoubleMatrix &,
			   DoubleMatrix &, DoubleMatrix &, 
			   DoubleVector  &, double &, double &, sBrevity &)
    const; 
};

ostream & operator <<(ostream &, const MssmSusy &);
istream & operator >>(istream &left, MssmSusy &s);
void setBetas(DoubleMatrix &, DoubleVector  &, DoubleVector  &, DoubleVector
	       &, DoubleVector  &);

//void inputLowe(const QedQcd & atMt, MssmSusy & s, double tanb);

inline void MssmSusy::setGaugeCoupling(int i, double f) { g(i) = f; }

inline void MssmSusy::setAllGauge(const DoubleVector & v) { 
  if (v.displayStart() != 1 || v.displayEnd() !=3) {
    cout << 
      "Initialising SUSY params gauge function with vector NOT 1..3\n" <<
      v;
    exit(1);
  }
  g = v; 
}

inline void MssmSusy::setSusyMu(double f) { smu = f; }
inline void MssmSusy::setTanb(double f) { tanb = f; }
inline DoubleVector MssmSusy::displayGauge() const { return g; }
inline double MssmSusy::displayGaugeCoupling(int i) const { 
  return g.display(i); 
}
inline double MssmSusy::displaySusyMu() const { return smu; }



#endif



